/*****************************************************************
 * Copyright (c) ParaSoft Corporation 1991.  All rights reserved.*
 * THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF ParaSoft       *
 * The copyright notice above does not evidence any              *
 * actual or intended publication of such source code.           *
 *****************************************************************/

/* Types of errors:
warning: just warning, will f77 afterwards.
fatal: fatal, will try to finish translation, but no f77 afterwards.
terminate: like fatal but will terminate inmediatelly.
comerr: to use when a system call gives an error, will print errno...
comerrnl: same as comerr but no line #, file info given.
syserr: internal compiler error. File name and line where it happened
        get printed automatically.
SYSERR: equivalent to syserr ((char*) 0).
NIYERR: for NIY.
*/

#include "extern.h"
#include <errno.h>

extern char *sys_errlist[];

char serr[1000];

int error_level = 0, num_errors = 0;

errsyn (str)
char *str;
{
	yyerror_str ("syntax error", str);
}

yyerror (s)  /*!! yytext not ok if unput */
char *s;
{
	yyerror_str (s, zztext);
}

yyerror_str (s, str)
char *s, *str;
{
	char *text;

	switch (str [0]) {

	case '\n':
		sprintf (serr, "%s: at or before '\\n'", s);
		break;

	case '{':
		sprintf (serr, "%s: at or before '('", s);
		break;

	case '~':
		sprintf (serr, "%s: at or before ','", s);
		break;

	default:
		if (!strncmp (str, "LPIMP", 5))
			text = "(";
		else
			text = str;

		sprintf (serr, "%s: at or before '%s'", s, text);
		break;

	}
	fatal (serr);
}

obsol (s)  /* obsolescent error */
char *s;
{
	char *str, *mess = "Obsolescent feature: ";

	if (flag_obsolescent == 0)
		return;

	str = (char*) U_malloc (strlen(s) + strlen(mess) + 1);
	sprintf (str, "%s%s", mess, s);

	output (str);
	U_free (str);

	if (error_level < ERR_OBSOLESCENT)
		error_level = ERR_OBSOLESCENT;
}

stand (s)  /* standard error */
char *s;
{
	char *str, *mess = "Extension: ";

	if (flag_standard == 0)
		return;

	str = (char*) U_malloc (strlen(s) + strlen(mess) + 1);
	sprintf (str, "%s%s", mess, s);

	output (str);
	U_free (str);

	if (error_level < ERR_OBSOLESCENT)
		error_level = ERR_OBSOLESCENT;
}

f90ni (s)  /* standard error */
char *s;
{
	char *str, *mess = "F90 feature not implemented: ";

	str = (char*) U_malloc (strlen(s) + strlen(mess) + 1);
	sprintf (str, "%s%s", mess, s);

	output (str);
	U_free (str);

	if (error_level < ERR_NOTIMPLEM)
		error_level = ERR_NOTIMPLEM;
}

f90ni_recursive ()
{
	f90ni ("recursive procedures");
}

f90ni_internal_procedures ()
{
	f90ni ("internal procedures");
}

f90ni_pointers ()
{
	f90ni ("pointers");
}

f90ni_modules ()
{
	f90ni ("modules");
}

f90ni_kind ()
{
	f90ni ("kind type parameter");
}

f90ni_uddtao ()
{
	f90ni ("user-defined data types and operators");
}

warning (s)  /* warning error */
char *s;
{
	char *str, *mess = "Warning: ";

	if (flag_w)
		return;

	str = (char*) U_malloc (strlen(s) + strlen(mess) + 1);
	sprintf (str, "%s%s", mess, s);

	output (str);
	U_free (str);

	if (error_level < ERR_WARNING)
		error_level = ERR_WARNING;
}

fatalnl (s)  /* fatal error, no file|line info given */
char *s;
{
	outputnl (s);

	if (error_level < ERR_FATAL)
		error_level = ERR_FATAL;
}

fatal (s)  /* fatal error */
char *s;
{
	output (s);

	if (error_level < ERR_FATAL)
		error_level = ERR_FATAL;
}

fatal_fl (s, file, line)  /* fatal error */
char *s;
char *file;
int line;
{
	output_fl (s, file, line);

	if (error_level < ERR_FATAL)
		error_level = ERR_FATAL;
}

terminate (s)  /* fatal error + exit() */
char *s;
{
	fatal (s);
	exit (1);
}

terminatenl (s)  /* fatal error + exit() */
char *s;
{
	fatalnl (s);
	exit (1);
}

comerrnl (s)  /* comerr + nl */
char *s;
{
	char *str;

	if (errno) {
		str = (char*) U_malloc (sizeof (char) * (
		              strlen (s) + strlen (sys_errlist[errno]) + 2 + 1));
		sprintf (str, "%s: %s", s, sys_errlist[errno]);
		errno = 0;
		fatalnl (str);
		U_free (str);
	}
	else
		fatalnl (s);
}

comerr (s)  /* fatal error, prints sys_errlist[errno], used when system call fails */
char *s;
{
	char *str;

	if (errno) {
		str = (char*) U_malloc (sizeof (char) * (
		              strlen (s) + strlen (sys_errlist[errno]) + 2 + 1));
		sprintf (str, "%s: %s", s, sys_errlist[errno]);
		errno = 0;
		fatal (str);
		U_free (str);
	}
	else
		fatal (s);
}

syserr_internal (file, line, s)  /* used by macros */
char *file;
int line;
char *s;
{
	char *str, *mess = "SYSTEM ERROR: ";

#ifdef RELEASE
	error_core ();
#else
	if (s == (char*) 0) {
		str = (char*) U_malloc (strlen(mess) + strlen(file) + 50 + 1);
		sprintf (str, "SYSTEM ERROR: at source file \"%s\", line %d", file, line);
	}
	else {
		str = (char*) U_malloc (strlen(s) + strlen(mess) + strlen(file) + 50 + 1);
		sprintf (str, "SYSTEM ERROR: at source file \"%s\", line %d: %s", file, line, s);
	}
	fatal (str);
	U_free (str);
	if (flag_coredump)
		U_coredump ();
#endif
}

syserr_internal_term (file, line, s)
char *file;
int line;
char *s;
{
	syserr_internal_term (file, line, s);
	exit (1);
}
/*********************************************************************************/
static outputnl (s)
char *s;
{
	static char *last_mess = CN;

	if (last_mess != CN)
		if (!strcmp (last_mess, s))
			return;

	if (last_mess != CN)
		U_free (last_mess);

	last_mess = (char*) U_malloc (strlen (s) + 1);
	strcpy (last_mess, s);

	fprintf(stderr,  "%s\n", s);

	num_errors++;
}

static output (s)
char *s;
{
	static char *last_mess = CN;
	static int last_line = 0;
	static char *last_file = CN;

	if (last_mess != CN)
		if (!strcmp (last_mess, s)
		&& !strcmp (last_file, input_filename)
		&&  last_line == real_zzlineno())
			return;

	if (last_mess != CN)
		U_free (last_mess);

	last_mess = (char*) U_malloc (strlen (s) + 1);
	strcpy (last_mess, s);
	last_file = (char*) U_malloc (strlen (input_filename) + 1);
	strcpy (last_file, input_filename);
	last_line = real_zzlineno ();

	fprintf(stderr,  "\"%s\", line %d: %s\n", input_filename, real_zzlineno (), s);

	num_errors++;
}

static output_fl (s, file, line)
char *s, *file;
int line;
{
	fprintf(stderr,  "\"%s\", line %d: %s\n", file, line, s);

	num_errors++;
}
/*********************************************************************************/

error_check ()  /* calls exit if error_level >= ERR_FATAL */
{
	if (!flag_ignore_fatal && error_level >= ERR_FATAL) {
		output_delete_output_file ();
		exit (1);
	}
}

error_init ()  /* just set errno to 0 */
{
	errno = 0;
}

char *error_s (num)
int num;
{
	if (num == 1)
		return "";
	else
		return "s";
}

error_core ()
{
	if (error_level >= ERR_FATAL)
		terminate ("cannot recover from earlier errors, exiting");
	else
		terminate ("INTERNAL COMPILER ERROR");
}

error_quit ()
{
}
